package handlers;

import com.rti.dds.domain.DomainParticipant;
import com.rti.dds.domain.DomainParticipantFactory;
import com.rti.dds.infrastructure.InstanceHandle_t;
import com.rti.dds.infrastructure.StatusKind;
import com.rti.dds.publication.Publisher;
import com.rti.dds.topic.Topic;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;

import schema.Alert;
import schema.AlertDataWriter;
import schema.AlertTypeSupport;


import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;

public class AlertHandler implements HttpHandler {
	
	DomainParticipant participant = null;
    Publisher publisher = null;
    Topic topic = null;
    AlertDataWriter writer = null;
	
	public AlertHandler() {
		initialize(0);
	}
	
	private void initialize(int domainId) {
		try {
            // --- Create participant --- //

            /* To customize participant QoS, use
            the configuration file
            USER_QOS_PROFILES.xml */

            participant = DomainParticipantFactory.TheParticipantFactory.
            create_participant(
                domainId, DomainParticipantFactory.PARTICIPANT_QOS_DEFAULT,
                null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (participant == null) {
                System.err.println("create_participant error\n");
                return;
            }        

            // --- Create publisher --- //

            /* To customize publisher QoS, use
            the configuration file USER_QOS_PROFILES.xml */

            publisher = participant.create_publisher(
                DomainParticipant.PUBLISHER_QOS_DEFAULT, null /* listener */,
                StatusKind.STATUS_MASK_NONE);
            if (publisher == null) {
                System.err.println("create_publisher error\n");
                return;
            }                   

            // --- Create topic --- //

            /* Register type before creating topic */
            String typeName = AlertTypeSupport.get_type_name();
            AlertTypeSupport.register_type(participant, typeName);

            /* To customize topic QoS, use
            the configuration file USER_QOS_PROFILES.xml */

            topic = participant.create_topic(
                "Alert",
                typeName, DomainParticipant.TOPIC_QOS_DEFAULT,
                null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (topic == null) {
                System.err.println("create_topic error\n");
                return;
            }
            
            // --- Create writer --- //

            /* To customize data writer QoS, use
            the configuration file USER_QOS_PROFILES.xml */

            writer = (AlertDataWriter)
            publisher.create_datawriter(
                topic, Publisher.DATAWRITER_QOS_DEFAULT,
                null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (writer == null) {
                System.err.println("create_datawriter error\n");
                return;
            }
            
		} catch (Exception e) {
			// TODO: handle exception
		}
	}

    public void handle(HttpExchange httpExchange) throws IOException {

        BufferedReader reader = new BufferedReader(new InputStreamReader(httpExchange.getRequestBody()));

        String line;
        String full = "";

        while ((line = reader.readLine()) != null) {
            full += line;
        }

        reader.close();

        //System.out.println(full);

        try {
            final JSONObject obj = new JSONObject(full);
            //System.out.println(obj.getString("state"));
            if (!obj.getString("state").equalsIgnoreCase("ok")) {
                System.out.println(obj.getString("message"));
                Alert instance = new Alert();
                instance.Alert = obj.getString("message");
                writer.write(instance, InstanceHandle_t.HANDLE_NIL);
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }

        String response = "ACK";
        httpExchange.sendResponseHeaders(200, response.length());

        OutputStream os = httpExchange.getResponseBody();
        os.write(response.getBytes());
        os.close();
    }

}
